@@ -22,7 +22,9 @@ from typing import TYPE_CHECKING
 import uuid
 
 from bleak.backends.device import BLEDevice
+from bleak.backends.scanner import AdvertisementData
 from bleak.exc import BleakError
+from bleak_retry_connector import retry_bluetooth_connection_error
 
 from aiohomekit.controller.abstract import AbstractDiscovery, FinishPairing
 from aiohomekit.model import CharacteristicsTypes, ServicesTypes
@@ -31,12 +33,7 @@ from aiohomekit.protocol import perform_pair_setup_part1, perform_pair_setup_par
 from aiohomekit.utils import check_pin_format, pair_with_auth
 
 from .bleak import AIOHomeKitBleakClient
-from .client import (
-    char_read,
-    char_write,
-    drive_pairing_state_machine,
-    retry_bluetooth_connection_error,
-)
+from .client import char_read, char_write, drive_pairing_state_machine
 from .connection import establish_connection
 from .manufacturer_data import HomeKitAdvertisement
 from .pairing import BlePairing
@@ -61,10 +58,12 @@ class BleDiscovery(AbstractDiscovery):
         controller: BleController,
         device: BLEDevice,
         description: HomeKitAdvertisement,
+        ble_advertisement: AdvertisementData,
     ) -> None:
         self.description = description
         self.controller = controller
         self.device = device
+        self.ble_advertisement = ble_advertisement
         self.client: AIOHomeKitBleakClient | None = None
         self._connection_lock = asyncio.Lock()
 
@@ -74,7 +73,7 @@ class BleDiscovery(AbstractDiscovery):
 
     @property
     def rssi(self) -> int | None:
-        return self.device.rssi if self.device else None
+        return self.ble_advertisement.rssi if self.ble_advertisement else None
 
     async def _ensure_connected(self):
         logger.debug(
@@ -201,8 +200,12 @@ class BleDiscovery(AbstractDiscovery):
         await char_write(self.client, None, None, char, iid, b"\x01")
 
     def _async_process_advertisement(
-        self, device: BLEDevice, description: HomeKitAdvertisement
+        self,
+        device: BLEDevice,
+        description: HomeKitAdvertisement,
+        ble_advertisement: AdvertisementData,
     ):
         """Update the device and description so we connect to the right place."""
         self.device = device
+        self.ble_advertisement = ble_advertisement
         self.description = description
