@@ -23,6 +23,15 @@ from aiohomekit.tlv8 import TLVStruct, tlv_entry, u8, u16, u128
 from .const import AdditionalParameterTypes
 
 
+class HAP_BLE_PROTOCOL_CONFIGURATION_REQUEST_TLV(enum.IntEnum):
+    GenerateBroadcastEncryptionKey = 0x01
+
+
+class HAP_BLE_CHARACTERISTIC_CONFIGURATION_REQUEST_TLV(enum.IntEnum):
+    kTLVHAPParamCharacteristicConfigurationProperties = 0x01
+    kTLVHAPParamCharacteristicConfigurationBroadcastInterval = 0x02
+
+
 class HAP_TLV(enum.IntEnum):
     # Additional Parameter Types for BLE (Table 6-9 page 98)
     kTLVHAPSeparator = 0x00
@@ -287,6 +296,12 @@ class Characteristic(TLVStruct):
             "perms": perms,
         }
 
+        if self.supports_broadcast_notify:
+            result["broadcast_events"] = True
+
+        if self.notifies_events_in_disconnected_state:
+            result["disconnected_events"] = True
+
         if self.data_type_str != "unknown":
             result["format"] = self.data_type_str
 
